import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

public class TCPClient
{
    public static final int NO_MORE_DATA = -1;
    public static final int TIME_OUT = -2;
    public static final int CONNECTION_TERMINATED = -3;
    public static final int SOCKET_EXCEPTION = -4;
    public static final int OK = 1;
    public static final int FAILED = 2;

    public TCPClient() {
        socket = null;
    }
    
    public TCPClient(Socket s) {
        socket = s;
    }

    public void close() {
        try {
            if (socket != null) {
                socket.close();
                socket = null;
            }
        }
        catch (IOException e) {
            System.out.println("TCPSocket.close: couldn't close the socket.\n");
        }
    }
    
    public int connectToServer(String dest, int port) {
        try {
            socket = new Socket(dest, port);
            return OK;
        }
        catch (UnknownHostException e) {
            System.out.println("Cannot resolve the destination host name: " + dest + "\n");
            socket = null;
        }
        catch (IOException e) {
            System.out.println("I/O error occurred while creating a socket connect\n");
            socket = null;
        }
        catch (SecurityException e) {
            System.out.println("Security manager doesn't allow making connection\n");
            socket = null;
        }
        return FAILED;
    }

    public void setTimeoutPeriod(int timeout) {
        timeoutPeriod = timeout;
    }
    
    /*public int read(byte[] buffer, int nbytes, boolean blocking) {
        if (socket != null) {
            int nread = 0;
            int prevTimeout;
            try {
                prevTimeout = socket.getSoTimeout();
                if (blocking) socket.setSoTimeout(timeoutPeriod);
                else socket.setSoTimeout(0);
            }
            catch (SocketException e) {
            	System.out.println("setting timeout failed.");
                return SOCKET_EXCEPTION;
            }

            try {
            	System.out.println("nbytes = " + nbytes);
                InputStream is = socket.getInputStream();
            	for (int i = 0; i < nbytes; i++) {
                    buffer[i] = (byte) is.read();
                    nread++;
                }
            }
            catch (IOException e) {
            	System.out.println("connection terminated.");            	
                return CONNECTION_TERMINATED;
            }
//		    catch (InterruptedIOException e) {
//                return TIME_OUT;
//            }
            finally {
                try {
                    socket.setSoTimeout(prevTimeout);
                }
                catch (SocketException e) {
                }
            }
            System.out.println("nread = " + nread);
            return nread;
        }
        return CONNECTION_TERMINATED;
    } */

    public int read(byte[] buffer, int nbytes, boolean blocking) {
        if (socket != null) {
            int nread = 0;
            int prevTimeout;
            try {
                prevTimeout = socket.getSoTimeout();
                if (blocking) socket.setSoTimeout(timeoutPeriod);
                else socket.setSoTimeout(0);
            }
            catch (SocketException e) {
            	System.out.println("setting timeout failed.");
                return SOCKET_EXCEPTION;
            }

            try {
                InputStream is = socket.getInputStream();
                int readBytes = 0;
                
                while (nbytes > 0) {
                    readBytes = is.read(buffer, nread, nbytes);
                    nread += readBytes;
                    nbytes -= readBytes;
                }
            }
            catch (IOException e) {
            	System.out.println("connection terminated.");
                return CONNECTION_TERMINATED;
            }
    /*		catch (InterruptedIOException e) {
                return TIME_OUT;
            }*/
            finally {
                try {
                    socket.setSoTimeout(prevTimeout);
                }
                catch (SocketException e) {
                }
            }
            System.out.println("nread = " + nread);
            return nread;
        }
        return CONNECTION_TERMINATED;
    }

    public int write(byte[] buffer, int nbytes, boolean blocking) {
        if (socket != null) {
            int nwrite = 0;
            int prevTimeout;
            try {
                prevTimeout = socket.getSoTimeout();
                if (blocking) socket.setSoTimeout(timeoutPeriod);
                else socket.setSoTimeout(0);
            }
            catch (SocketException e) {
                return SOCKET_EXCEPTION;
            }

            try {
                if (blocking) socket.setSoTimeout(timeoutPeriod);
                else socket.setSoTimeout(0);
            }
            catch (SocketException e) {
                return SOCKET_EXCEPTION;
            }

            try {
                OutputStream os = socket.getOutputStream();
                os.write(buffer, 0, nbytes);
            }
            catch (IOException e) {
                return CONNECTION_TERMINATED;
            }
    /*		catch (InterruptedIOException e) {
                return TIME_OUT;
            }*/
            finally {
                try {
                    socket.setSoTimeout(prevTimeout);
                }
                catch (SocketException e) {
                }
            }

            return nbytes;
        }
        return CONNECTION_TERMINATED;
    }

    protected int timeoutPeriod;
    protected Socket socket;
}
